load File.dirname(__FILE__) + "/../test_helper.rb"
# require "httpclient" #  using rest-client is simpler
require "rest-client"
require "json" 

# Sample Restful service:  https://dummy.restapiexample.com/
# Sometimes return 429 Too Many Requests

describe "REST WebService" do
  include TestHelper

  def api_host
    return "https://dummy.restapiexample.com/api/v1"
  end

  it "REST - List all records (JSON Response)" do
    require "rest-client"
    list_rest_url = api_host + "/employees"
    puts list_rest_url
    resp = RestClient.get(list_rest_url)
    File.open("C:/temp/rest.xml", "w").puts(resp.body) if RUBY_PLATFORM =~ /mingw/
    json_obj = JSON.parse(resp.body)
    expect(json_obj["data"].size).to be > 10
    
    # process data from a list of records, show off Ruby
    all_employee_ages = json_obj["data"].collect { |x| x["employee_age"].to_i }
    puts all_employee_ages
    avg_employee_age = all_employee_ages.sum(0.0) / all_employee_ages.size
    puts("Average Employee Age: #{avg_employee_age}")
  end

  it "REST - Get a record" do
    employee_id = 1
    get_rest_url = "#{api_host}/employee/#{employee_id}"
    puts get_rest_url
    resp = RestClient.get(get_rest_url)
    # puts resp.body
    expect(resp.body).to include("Successfully! Record has been fetched")
    #specific verification
    json_obj = JSON.parse(resp.body)
    expect(json_obj["data"]["employee_name"]).to eq("Tiger Nixon")
  end

  it "Create Record (Post JSON)" do
    # using the Get request to get sample JSON
    json_request = <<END_OF_MESSAGE
{
  "name": "test",
  "age":  "24",
  "salary": "34567",
}
END_OF_MESSAGE
    ws_url = api_host + "/create"
    response = RestClient.post(ws_url, json_request)
    puts "=>|#{response.body}|"
    expect(response.body).to include("Successfully! Record has been added.")
  end

  it "Update Record (PUT JSON)" do
    # use HTTP PUT or Patch to update an existing resource
    json_request = <<END_OF_MESSAGE
{
  "name": "test",
  "age":  "24",
  "salary": "34567",
}
END_OF_MESSAGE

    json_request.gsub!("24", "60")
    update_url = api_host + "/update/1"
    response = RestClient.put(update_url, json_request)
    puts response.body
    expect(response.body).to include("Successfully! Record has been updated.")
  end

  it "REST - Delete a record" do
    delete_rest_url = api_host + "/delete/2"
    resp = RestClient.delete(delete_rest_url)
    puts resp.body
    expect(resp.body).to include("Record has been deleted")
  end

end
