load File.dirname(__FILE__) + "/../test_helper.rb"

describe "SOAP Testing" do
  include TestHelper

  before(:all) do
  end

  after(:all) do
  end
  
  it "SOAP with sample XML with HTTPClient" do
    request_xml = <<END_OF_MESSAGE
<?xml version="1.0" encoding="utf-8"?>
<soap12:Envelope xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:xsd="http://www.w3.org/2001/XMLSchema" xmlns:soap12="http://www.w3.org/2003/05/soap-envelope">
  <soap12:Body>
    <CelsiusToFahrenheit xmlns="https://www.w3schools.com/xml/">
      <Celsius>20</Celsius>
    </CelsiusToFahrenheit>
  </soap12:Body>
</soap12:Envelope>
END_OF_MESSAGE

    # http = Net::HTTP.new('www.w3schools.com', 443)
    uri = URI.parse("https://www.w3schools.com/xml/tempconvert.asmx")
    http = Net::HTTP.new(uri.host, uri.port)
    http.use_ssl = true
    resp, data = http.post(uri.path, request_xml,
                           { "SOAPAction": "https://www.w3schools.com/xml/CelsiusToFahrenheit",
                             "Content-Type": "text/xml",
                             "Host": "www.w3schools.com" })
    puts resp.body
    expect(resp.code).to eq("200") # OK
    # debug resp.body
    # resp.each { |key, val| debug(key + ' = ' + val) }
    expect(resp.body).to include("<CelsiusToFahrenheitResult>68</CelsiusToFahrenheitResult>")
  end


  it "SOAP with dynamic request data with HTTPClient" do
    template_erb_file = File.expand_path("../../testdata/c_to_f.xml.erb", __FILE__)
    template_erb_str = File.read(template_erb_file)
    @degree = 30 # changeable in your test script
    request_xml = ERB.new(template_erb_str).result(binding)

    uri = URI.parse("https://www.w3schools.com/xml/tempconvert.asmx")
    http = Net::HTTP.new(uri.host, uri.port)
    http.use_ssl = true
    resp, data = http.post(uri.path, request_xml, {
      "SOAPAction" => "https://www.w3schools.com/xml/CelsiusToFahrenheit",
      "Content-Type" => "text/xml",
      "Host" => "www.w3schools.com",
    })
    expect(resp.code).to eq("200") # OK
    puts resp.body
    expect(resp.body).to include("<CelsiusToFahrenheitResult>86</CelsiusToFahrenheitResult>")
  end
  
  it "SOAP with Dynamic Request Data with HTTParty" do
    template_erb_str = File.read(template_file("c_to_f.xml.erb"))
    @degree = 30 # changeable in your test script
    request_xml = ERB.new(template_erb_str).result(binding)
    
    #   or quicker way (defined in test_helper)
    # request_xml = generate_from_template("c_to_f.xml.erb") 
    
    resp = HTTParty.post("https://www.w3schools.com/xml/tempconvert.asmx", 
      :body => request_xml,
      :headers => {
        "SOAPAction" => "https://www.w3schools.com/xml/CelsiusToFahrenheit",
        "Content-Type" => "text/xml",
        "Host" => "www.w3schools.com",
       })
    expect(resp.code).to eq(200) # OK
    puts resp.body
    expect(resp.body).to include("<CelsiusToFahrenheitResult>86</CelsiusToFahrenheitResult>")
  end
  
  it "Parse SOAP Response XML after stripping namespace" do
    xml = '<?xml version="1.0" encoding="utf-8"?><soap:Envelope xmlns:soap="http://schemas.xmlsoap.org/soap/envelope/" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:xsd="http://www.w3.org/2001/XMLSchema"><soap:Body><CelsiusToFahrenheitResponse xmlns="https://www.w3schools.com/xml/"><CelsiusToFahrenheitResult>86</CelsiusToFahrenheitResult></CelsiusToFahrenheitResponse></soap:Body></soap:Envelope>'
    require "nokogiri"
    xml_doc = Nokogiri.parse(xml)
    xml_doc.remove_namespaces!

    # debug xml_doc.to_s #
    node = "//CelsiusToFahrenheitResponse/CelsiusToFahrenheitResult"
    expect(xml_doc.xpath(node).text).to eq("86")
  end
  
  
end
