# Inlcude this file to support better test exeuction in TestWise.
#

# add supporting  puts() in rspec scripts
require 'socket'
require 'pstore'
require 'timeout'

# the maximum characters send to TestWise
unless defined?(MAX_MESSAGE_LENGTH)
  MAX_MESSAGE_LENGTH = 8192
end

$process_id = nil

module Cucumber
  module Glue
 
    # overwrites to 
    module ProtoWorld
      def log(*messages)        
        messages.each { |message| attach(message.to_s.dup, 'text/x.cucumber.log+plain') }
        
        #
        write_to_console(messages.to_s)
      end
      
    end
  end
end

module TestWiseRuntimeSupport

    def self.included(mod)
    # Only for Windows, some environment variables need to be set to
    # get enable network communication between test execution and TestWise (not necessary for Mac)
    
    # NOTE: if your windows installation is different (such as on D:), please update below accordingly
      if RUBY_PLATFORM =~ /mingw/
        unless ENV["SYSTEMROOT"] && Dir.exist?(ENV["SYSTEMROOT"])        
          ENV["SYSTEMROOT"] = "C:\\WINDOWS"
        end
      
        unless ENV["SYSTEMDRIVE"] && Dir.exist?(ENV["SYSTEMDRIVE"])        
          ENV["SYSTEMDRIVE"] = "C:"
        end
      
        unless ENV["windir"] && Dir.exist?(ENV["windir"])        
          ENV["windir"] = "C:\\WINDOWS"
        end

        # the %TEMP% is needed for Selenium Driver, in case not set, using default one
        unless ENV["TEMP"] && Dir.exist?(ENV["TEMP"])
          ENV["TEMP"] = "C:\\TEMP"
          ENV["TMP"] = "C:\\TEMP"
        end
      end
      
      $TESTWISE_TRACE_PORT = nil
      begin
        the_testwise_port_file = File.join(File.dirname(__FILE__), "testwise.port")
        if File.exist?(the_testwise_port_file)
          self.log_comm("[COMM] Exists testwise.port file, included in #{mod}")
          the_file_content = File.read(the_testwise_port_file).strip
          if the_file_content.to_i > 1024 &&  the_file_content.to_i < 65500
            $TESTWISE_TRACE_PORT = the_file_content.to_i 
            self.log_comm("[COMM] $TESTWISE_TRACE_PORT is set from file! #{$TESTWISE_TRACE_PORT}")
          end
        end
      rescue => e
        self.log_comm("[COMM] Error on trying to load testwise.port: #{e}")          
        $TESTWISE_TRACE_PORT = nil
      end
      
      if $TESTWISE_TRACE_PORT.nil? && ENV["TESTWISE_TRACE_PORT"]
         the_env_port = ENV["TESTWISE_TRACE_PORT"].to_i 
         if the_env_port > 1024 &&  the_env_port < 65500
           $TESTWISE_TRACE_PORT = the_env_port
           self.log_comm("[COMM] $TESTWISE_TRACE_PORT is set from env var! #{$TESTWISE_TRACE_PORT}") 
         else           
           self.log_comm("[COMM] Invalid env variable port number: #{the_env_port}")          
         end
       end
      
       if $TESTWISE_TRACE_PORT.nil?
         self.log_comm("[COMM] : Tried file and env but both failed,  $TESTWISE_TRACE_PORT is set to use default")                   
         $TESTWISE_TRACE_PORT = 7535 # using the default
       end
      
      
if  $process_id.nil? # RUBY_PLATFORM =~ /mingw/ && 
      $process_id = Process.pid
      begin
        the_message = "PID" + "|#{Process.pid}"   
        testwise_port = $TESTWISE_TRACE_PORT
        testwise_socket = Socket.new(Socket::AF_INET, Socket::SOCK_STREAM, 0)
        testwise_socket.connect(Socket.pack_sockaddr_in(testwise_port, '127.0.0.1'))
        testwise_socket.puts(the_message)
        testwise_socket.close        
      rescue => e
      end      
end
    
    end

    def debugging?
      if ENV["RUN_IN_TESTWISE"].to_s == "true" && ENV["TESTWISE_RUNNING_AS"] == "test_case"
        return true
      end
      return $TESTWISE_DEBUGGING && $TESTWISE_RUNNING_AS == "test_case"
    end

    alias :oldputs :puts    
    def write_to_console(message)       
#      Thread.pass
#      if (ENV["RUN_IN_TESTWISE"].to_s == "true" || $RUN_IN_TESTWISE) && message
       if ENV["RUN_IN_TESTWISE"].to_s == "true" 
          begin
            the_sent_msg = message.to_s
            if the_sent_msg.size > MAX_MESSAGE_LENGTH
              the_sent_msg = the_sent_msg[0..MAX_MESSAGE_LENGTH] + "..."
            end
            connect_to_testwise("OUTPUT",  the_sent_msg + "\r\n")
          rescue
            oldputs message
          end
        else
          oldputs(message)
        end        
    end

    alias debug write_to_console
    alias puts write_to_console
    alias log write_to_console

    # Support of TestWise to ajust the intervals between keystroke/mouse operations
    def operation_delay
      begin
        $TESTWISE_OPERATION_DELAY = ENV["TESTWISE_OPERATION_DELAY"] if ENV["TESTWISE_OPERATION_DELAY"]
        if $TESTWISE_OPERATION_DELAY && $TESTWISE_OPERATION_DELAY.to_i > 0.0001 &&
          $TESTWISE_OPERATION_DELAY.to_i < 300  then # max 30 seconds
            Thread.pass
            sleep($TESTWISE_OPERATION_DELAY.to_i / 10.0)
        end

        while (ENV["TESTWISE_PAUSE"] && ENV["TESTWISE_PAUSE"].to_s == "true")  || $TESTWISE_PAUSE
          Thread.pass
          write_to_console("Paused, waiting ...")
          sleep 1
        end
      rescue => e
        puts "Error on delaying: #{e}"
        # ignore
      end
    end

    def notify_screenshot_location(image_file_path)
      connect_to_testwise("  SHOT", image_file_path)
    end

      # NOTE
      #
      #  This one works, not selenium_...extension
      
      # find out the line (and file) the execution is on, and notify iTest via Socket
      def dump_caller_stack        
        return unless (ENV["TESTWISE_TRACE_EXECUTION"].to_s == "true" || $TESTWISE_TRACE_EXECUTION)

        begin 
          Timeout::timeout(1) do
            start_time = Time.now
            begin
              trace_lines = []
              trace_file = nil
              found_first_spec_reference = false
              caller_stack= caller
         
              caller_stack.delete_if {|x|
                x.include?("/gems/") || 
                x.include?("/runtime") || 
                x.include?("/lib/cucumber/")  || 
                x.include?("spec_helper.rb")
              }

              is_rspec = caller_stack.any?{|x| x.include?("/bin/rspec") }
              is_cucumber = caller_stack.any?{|x| x.include?("bin/cucumber")}
              caller_stack.delete_if {|x| x.include?("/bin/") }

              # fio = File.open("/tmp/trimmed-testwise-support-mssage.log", "a")
              # fio.puts("is_rspec: #{is_rspec} | is_cucumber: #{is_cucumber} " +  caller_stack.join("\n") + "\n\n\n")
          
              caller_stack.each_with_index do |position, idx|
                next unless position =~ /\A(.*?):(\d+)/
                trace_file = $1
                # end_with? is faster than regex 
                if is_rspec &&  (trace_file.end_with?("_spec.rb") || trace_file.end_with?("_test.rb") ) 
                  found_first_spec_reference = true
                  trace_lines << position
                  break
                elsif is_cucumber && trace_file.end_with?(".feature")
                  found_first_spec_reference = true
                  trace_lines << position
                  break
                end
                trace_lines << position
                break if trace_lines.size > 20
                # TODO: send multiple trace to be parse with pages.rb
                # break if trace_file =~ /example\/example_methods\.rb$/ or trace_file =~ /example\/example_group_methods\.rb$/ or trace_file =~ /driver\.rb$/ or trace_file =~ /timeout\.rb$/ # don't include rspec or ruby trace
              end

              #  (trace_file.include?("_spec.rb") || trace_file.include?("_rwebspec.rb") || trace_file.include?("_test.rb") || trace_file.include?("_cmd.rb"))
              if !trace_lines.empty?
                connect_to_testwise(" TRACE", trace_lines.reverse.join("|"))
              end

            rescue => e
              puts "failed to capture log: #{e}"
            ensure
              # fio = File.open("/tmp/dump_caller_stack_timing.log", "a")
              # fio.puts("dump_caller_stack v1, Time => #{Time.now - start_time}")
            end
          end
        rescue Timeout::Error
          # took too long, ignore the trace
        end
    
      end

      def get_testwise_db_file()
        return ENV["TESTWISE_DB_FILE"]
      end      
      
      def appium_session_store_file 
        pstore_file = File.expand_path File.join(File.dirname(__FILE__), "tmp", "appium-session.pstore")        
      end
      
      def retrieve_last_appium_session_id
        pstore_file =  appium_session_store_file    
        if File.exist?(pstore_file)
          wd_session_pstore = PStore.new(pstore_file)
          the_last_appium_session = wd_session_pstore.transaction { wd_session_pstore[:appium_session_id] }
          return the_last_appium_session
        end
        return nil
      end
      
      def store_appium_session_id(sid)
        appium_session_store = PStore.new(appium_session_store_file)
        appium_session_store.transaction do
          appium_session_store[:appium_session_id] = sid
          appium_session_store.commit
        end        
      end
      
      
      def webdriver_session_store_file 
        pstore_file = File.expand_path File.join(File.dirname(__FILE__), "tmp", "wdsession.pstore")        
        return pstore_file 
      end
      
      def last_session_browser_debugging_port
        pstore_file =  webdriver_session_store_file
        if File.exist?(pstore_file)
          wd_session_pstore = PStore.new(pstore_file)
          last_used_port = wd_session_pstore.transaction { wd_session_pstore[:webdriver_debugging_port] }
          return last_used_port
        end
        return nil
      end
            
      def get_browser_debugging_port
        webdriver_debugging_port = local_available_port();
        if (webdriver_debugging_port < 0) 
          # try again
          sleep 0.5
          webdriver_debugging_port = local_available_port();        
        end 
        
        if (webdriver_debugging_port < 0)
          # still not found, which is highly unlikely, try alternativ eway
          webdriver_debugging_port = local_available_port_pstore
        end
        
        wd_session_pstore = PStore.new(webdriver_session_store_file)
        wd_session_pstore.transaction do
          wd_session_pstore[:webdriver_debugging_port] = webdriver_debugging_port || default_port
          wd_session_pstore.commit
        end        
        ENV["BROWSER_DEBUGGING_PORT"] = webdriver_debugging_port.to_s
        return webdriver_debugging_port
      end
      
      def port_open?(ip, port, seconds=1)
        Timeout::timeout(seconds) do
          begin
            TCPSocket.new(ip, port).close
            true
          rescue Errno::ECONNREFUSED, Errno::EHOSTUNREACH
            false
          end
        end
      rescue Timeout::Error
        false
      end

      # The old way: based on saved  last port number and increment
      # 
      def local_available_port_pstore
          default_port = 19218
          webdriver_debugging_port = nil

          last_used_port = last_session_browser_debugging_port
          if last_used_port && last_used_port > 1  && last_used_port < 65500 
            webdriver_debugging_port = last_used_port + 1
          end
        
          webdriver_debugging_port ||= default_port
          return webdriver_debugging_port
      end
      
      # return a port that is available
      def local_available_port(port_range = []) 
        if port_range.nil? || port_range.empty? 
          port_range = []
          20.times do 
            port_range << rand(19318..19898)
          end
          port_range.uniq!
        end
  
        port_range.each do |port|
          next if port_open?("127.0.0.1", port, 0.5)
          return port
        end
        return -1
      end
      
      ## save the webdriver to a pstore (under TestWise)
      #
      # Currently not used, the chrome debugging port approach is better
      def save_webdriver_session
        require "pstore"
        executor_url = driver.send(:bridge).send(:http).send(:server_url).to_s
        session_id = driver.send(:bridge).session_id.to_s
    
        begin
          pstore_file = File.join(Dir.dirname(__FILE__), "..", "tmp", "wdsession.pstore")
          if session_id && executor_url && executor_url.size() > 5    
            wd_session_pstore = PStore.new("webdriver-session.pstore")
            wd_session_pstore.transaction do
                wd_session_pstore["executor_url"] = executor_url
                wd_session_pstore["session_id"] = session_id
            end
          end
        rescue => e
                    
        end
      end

      
      ## NOT USED
      #
      def save_webdriver_session_sqlite3
        require "sqlite3"
        executor_url = driver.send(:bridge).send(:http).send(:server_url).to_s
        session_id = driver.send(:bridge).session_id.to_s

        if session_id && executor_url && executor_url.size() > 5
          puts("[SAVE] WDURL: " + executor_url + ", session id: " + session_id)
          testwise_db_file = get_testwise_db_file()

          if testwise_db_file && File.exist?(testwise_db_file)
            db = SQLite3::Database.open testwise_db_file
            db.execute("UPDATE TEST_EXECUTIONS SET WEBDRIVER_SESSION_ID='" + session_id + "', WEBDRIVER_URL='" + executor_url + "' WHERE  id = (SELECT MAX(id) FROM TEST_EXECUTIONS)")
            db.close
          end
        end
      end

      def reuse_last_browser_session()
        testwise_db_file = get_testwise_db_file()
        if testwise_db_file && File.exist?(testwise_db_file)
          db = sqlite3.connect(testwise_db_file)
          db.query("SELECT WEBDRIVER_SESSION_ID, WEBDRIVER_URL FROM TEST_EXECUTIONS WHERE id = (SELECT MAX(id) FROM TEST_EXECUTIONS)")
          records = cursor.fetchall()
          session_id = records[0][0]
          executor_url = records[0][1]
          db.close
          puts("RETRIEVE WDURL: " + executor_url)
          return cls.attach_to_session(executor_url, session_id)
        end
      end

    # debugging only on Mac
    def self.log_comm(msg)
      the_file_path = File.join(File.dirname(__FILE__), "testwise-comm.log")
      fio = File.open(the_file_path, "a")
      fio.puts("#{Time.now} -> #{msg}")
      fio.flush
      fio.close
    end
    
    def connect_to_testwise(message_type, body, expect_response=false)
      Thread.pass
      begin
        the_message = message_type + "|" + body
  
        the_message = the_message[0.. 8000]
        if message_type.strip == "OUTPUT" && @last_message == the_message then # ignore the message same as preivous one
            return
        end
        
        testwise_port = $TESTWISE_TRACE_PORT
        testwise_socket = Socket.new(Socket::AF_INET, Socket::SOCK_STREAM, 0)
        testwise_socket.connect(Socket.pack_sockaddr_in(testwise_port, '127.0.0.1'))
        # ZZ: seems no difference when setting encoding
        # testwise_socket.set_encoding 'ASCII-8BIT'
        testwise_socket.set_encoding 'UTF-8'        
        # log_comm("send socket message: " + the_message)

        testwise_socket.puts(the_message)
        @last_message = the_message
        
        testwise_response =  nil
        if (expect_response)
          # this blocks (or much slower)
          # testwise_response = socket.read(128)
        end
                
        testwise_socket.close
        return testwise_response
      rescue => e
        
        begin
          the_file_path = File.join(File.dirname(__FILE__), "testwise-comm.log")
          fio = File.open(the_file_path, "a")
          fio.puts("connect_to_testwise error: #{e}")
          fio.flush
          fio.close
        rescue => e2
        end        
        return nil;
      end
    end

    def encode_string(s)
        begin
            [s.length].pack('n') + Iconv.conv('utf-16be', 'utf-8', s)
        rescue
            [s.length].pack('n') + s.encode('utf-16be').force_encoding('ASCII-8BIT')
        end
    end
    
  def check_for_pause_via_socket

    json_data = fetch_debug_server_status()

    if json_data
        
        if json_data["testwise_ready_to_pause"]
          # Already executed the the line immedately above,
          # give some buffer time for TW to set $TESTWISE_PAUSE flag
          sleep 0.5
        end

        # If the executed line no change, ignore
        while json_data["testwise_pause"] || $TESTWISE_PAUSE
          Thread.pass
          write_to_console("[selenium_webdriver] Paused, waiting ...")
          sleep 1
          json_data = fetch_debug_server_status
        end

    end
    
  end


  def check_for_pause(check_interval = 1)
    runtime_tmp_dir = File.join(File.dirname(__FILE__), "tmp")
    pid_file = File.join(runtime_tmp_dir, "execution.pid")
    pid = File.read(pid_file)  rescue nil
    return if pid.nil? || pid.empty?

    # enforce check on interval
    check_interval = 1 unless check_interval && check_interval > 0.5 && check_interval < 10
    
    pause_file = File.join(runtime_tmp_dir, "#{pid.strip}.pause")
    pause_file_exists = File.exist?(pause_file)
    while pause_file_exists
      Thread.pass
      write_to_console("[selenium_webdriver] Paused, waiting ...")
      sleep check_interval
      pause_file_exists = File.exist?(pause_file)
    end      
  end



  def get_current_test_execution_line

    begin        
      caller_stack = caller        
      caller_stack.delete_if {|x| !x.include?("_spec.rb") && !x.include?(".feature") && !x.include?("._steps.rb") }

      the_line = caller_stack[0] 
      array = []      

      if the_line =~ /\s*(.*):(\d+):in\s/
        test_script_file = $1
        line_number = $2
        array << [ test_script_file, line_number]
        return test_script_file, line_number          
      end 
      return nil, nil
    rescue => e
      return nil, nil
    end

  end
  
  def fetch_debug_server_status    
      test_script_file, line_number = get_current_test_execution_line
      return "{}" if test_script_file.nil?     
      resp = nil
      begin
        testwise_breakpoints_json = connect_to_testwise("CHECK_BREAKPOINT", "#{test_script_file}|#{line_number}".strip, true)
        # puts testwise_breakpoints_json
          # READ DOES NOT WORK
          # File.open("/Users/zhimin/tmp/socket.log", "a").puts(resp);
          
      rescue => e
        puts("Failed: #{e} ");
      ensure
        # ensure socket is closed
      end
      return resp
      #connect_to_testwise(" DEBUG", "");
      # return nil
=begin
      json_data = nil
    begin
        testwise_port = ENV["TESTWISE_TRACE_PORT"].to_i || $TESTWISE_TRACE_PORT
        if testwise_port < 1024
            testwise_port = 7535 # default
        end

        server = TCPSocket.open("127.0.0.1", testwise_debug_server_port)
        server.puts("debug_status")
        msg = server.gets.chomp
        server.close
        json_data = JSON.parse(msg)
        # puts "[DEBUG] debug server response #{json_data.inspect}" unless $using_rubyw
    rescue => e
    end
    return json_data
=end
  end

end



module TestWise

  class Runtime

      def self.load_watir_support
        load "#{File.dirname(__FILE__)}/watir_testwise_extension.rb"
      end
      
      def self.load_webdriver_support
        load "#{File.dirname(__FILE__)}/selenium_webdriver_testwise_extension.rb"
      end
      
      
  end

end
